#!/bin/bash -x
#SBATCH --job-name=merger
#SBATCH --partition=cpuq
#SBATCH --account=cpuq
#SBATCH --requeue
#SBATCH --nodes=1
#SBATCH --ntasks-per-node=40
#SBATCH --export=ALL
#SBATCH --time=24:00:00
#SBATCH --array 6-10
#SBATCH --mail-type=ALL
#SBATCH --mail-user=jwschwab@ucsc.edu

if [ -n "${SLURM_SUBMIT_DIR}" ]; then
   cd ${SLURM_SUBMIT_DIR}
   module load mesasdk/20.3.1
   export MESA_DIR=${DATA_DIR}/mesa-r12778
   export OMP_NUM_THREADS=36
else
    # assume you're already there
    SLURM_SUBMIT_DIR=$(pwd)
fi

if [ -z "${SLURM_ARRAY_TASK_ID}" ]; then
    SLURM_ARRAY_TASK_ID=1
fi

# rebuild MESA
# ./clean
# ./mk

# make sure there's a place for the movies and output
mkdir -p movies
mkdir -p output

# define a function to run a named model
do_one() {

    # get the relevant line from the batch file
    read ID INLIST_IO INLIST_MDOT INLIST_ROT INLIST_OTHER <<< $(sed "${1}q;d" < $2)

    # use the main inlist in the submit directory
    export MESA_INLIST=${SLURM_SUBMIT_DIR}/inlist

    # make a temporary directory
    TMPDIR=$(mktemp -d)
    cd ${TMPDIR}

    # setup inlists via soft links
    ln -sf ${SLURM_SUBMIT_DIR}/inlist_merger .
    ln -sf ${SLURM_SUBMIT_DIR}/inlists/${INLIST_IO} inlist_io
    ln -sf ${SLURM_SUBMIT_DIR}/inlists/${INLIST_MDOT} inlist_mass_loss
    ln -sf ${SLURM_SUBMIT_DIR}/inlists/${INLIST_ROT} inlist_rotation
    ln -sf ${SLURM_SUBMIT_DIR}/inlists/${INLIST_OTHER} inlist_other
    ln -sf ${SLURM_SUBMIT_DIR}/inlist_pgstar .
    
    # softlink in some more stuff
    ln -sf ${SLURM_SUBMIT_DIR}/history_columns.list .
    ln -sf ${SLURM_SUBMIT_DIR}/profile_columns.list .
    ln -sf ${SLURM_SUBMIT_DIR}/CO-table-with-metals.dat .

    ln -sf ${SLURM_SUBMIT_DIR}/models .
    
    # place for output
    mkdir -p ${SLURM_SUBMIT_DIR}/output/${ID}
    ln -sf ${SLURM_SUBMIT_DIR}/output/${ID} LOGS

    # run MESA
    ${SLURM_SUBMIT_DIR}/star

    # copy final output
    END_NAME=${INLIST_IO#inlist_}_terminate
    cp ${SLURM_SUBMIT_DIR}/models/${END_NAME}.mod ${SLURM_SUBMIT_DIR}/output/${ID}
    cp ${SLURM_SUBMIT_DIR}/models/${END_NAME}.data ${SLURM_SUBMIT_DIR}/output/${ID}

    # make movie
    DATE=$(date +%F)
    if [ -d png ]; then
	images_to_movie 'png/grid1*.png' ${SLURM_SUBMIT_DIR}/movies/${ID}-${DATE}.mp4
    fi

    cd -
    rm -rf ${TMPDIR}
    
}

#do_one ${SLURM_ARRAY_TASK_ID} test.batch
#do_one ${SLURM_ARRAY_TASK_ID} merger.batch
#do_one ${SLURM_ARRAY_TASK_ID} mdot.batch
#do_one ${SLURM_ARRAY_TASK_ID} section-He.batch
do_one ${SLURM_ARRAY_TASK_ID} merger2.batch
