      subroutine read_kasen_CO

        use interp_2d_lib_sg

        integer :: funit
        integer :: ii, jj
        real :: logt, logr, logd, logkap
        integer :: ierr
        
        ! read tables
        open(newunit=funit, file = "CO-table-with-metals.dat")
        do ii = 1, kk_nt
           do jj = 1, kk_nr
              read(funit, *) logt, logr, logd, logkap
              kk_k(jj,ii) = logkap
              kk_r(jj) = logr
           end do
           kk_t(ii) = logt
        end do
        close(funit)
        
        ! set up pointers
        allocate(wk(1:4, 1:kk_nr, 1:kk_nt), wk1(1:4*kk_nr*kk_nt))
        wk(1:4, 1:kk_nr, 1:kk_nt) => wk1(1:4*kk_nr*kk_nt)
        
        ! put tables in work array
        wk(1, 1:kk_nr, 1:kk_nt) = kk_k(1:kk_nr, 1:kk_nt)
        
        ! construct interpolant
        call interp_mkbicub_sg(kk_r, kk_nr, kk_t, kk_nt, wk1, kk_nr, &
             ibcxmin,bcxmin,ibcxmax,bcxmax, & 
             ibcymin,bcymin,ibcymax,bcymax, & 
             ilinx,iliny,ierr)
        
        if (ierr .ne. 0) stop "Failed while constructing interpolant"

        write(*,*) 'done reading'
        
      end subroutine read_kasen_CO
        
        
      subroutine kasen_CO(log10_rho, log10_T, kap, dln_kap_dlnRho, dln_kap_dlnT, ierr)

         use interp_2d_lib_sg

         real(dp), intent(in) :: log10_rho
         real(dp), intent(in) :: log10_T
         real(dp), intent(out) :: kap
         real(dp), intent(out) :: dln_kap_dlnRho
         real(dp), intent(out) :: dln_kap_dlnT
 
         integer :: ierr

         logical :: clipped

         real :: logd, logt, logr
 
         integer :: ict(6)
         real :: fval(6)
 
         logd = real(log10_Rho)
         logt = real(log10_T)
         logr = logd - 3*logT + 18

         clipped = .false.

         if ((logr .gt. maxval(kk_r)) .or. (logr .lt. minval(kk_r))) then 
            !write(*,*) "Off table in R: ", maxval(kk_r), "logr = ", logr, minval(kk_r)
            !ierr = -1
            !return
            clipped = .true.
            logr = min(max(logr, minval(kk_r)), maxval(kk_r))
         end if
         if ((logt .gt. maxval(kk_t)) .or. (logt .lt. minval(kk_t))) then
            !write(*,*) "Off table in T: ", maxval(kk_t), "logt = ", logt, minval(kk_t)
            !ierr = -1
            !return
            clipped = .true.
            logt = min(max(logt, minval(kk_t)), maxval(kk_t))
         end if
 
         ! get f, df/dx, df/dy
         ict(1:3) = 1
         ict(4:6) = 0
         
         call interp_evbicub_sg(logr,logt,kk_r,kk_nr,kk_t,kk_nt,&
              ilinx,iliny,wk1,kk_nr,ict,fval,ierr)
 
         kap = 10d0 ** fval(1)
         dln_kap_dlnRho = fval(2)
         dln_kap_dlnT = fval(3) - 3.0 * fval(2)

         if (clipped) then
            dln_kap_dlnRho = 0
            dln_kap_dlnT = 0
         end if
         
      end subroutine kasen_CO
      
        
      subroutine cold_CO_kap_get( &
            id, k, handle, zbar, X, Z, Zbase, XC, XN, XO, XNe, &
            log10_rho, log10_T, species, chem_id, net_iso, xa, &
            lnfree_e, d_lnfree_e_dlnRho, d_lnfree_e_dlnT, &
            frac_Type2, kap, dln_kap_dlnRho, dln_kap_dlnT, ierr)

         use kap_lib, only: kap_get
        
         ! INPUT
         integer, intent(in) :: id ! star id if available; 0 otherwise
         integer, intent(in) :: k ! cell number or 0 if not for a particular cell         
         integer, intent(in) :: handle ! from alloc_kap_handle
         real(dp), intent(in) :: zbar ! average ion charge
         real(dp), intent(in) :: X, Z, Zbase, XC, XN, XO, XNe ! composition    
         real(dp), intent(in) :: log10_rho ! density
         real(dp), intent(in) :: log10_T ! temperature
         real(dp), intent(in) :: lnfree_e, d_lnfree_e_dlnRho, d_lnfree_e_dlnT
         ! free_e := total combined number per nucleon of free electrons and positrons

         integer, intent(in) :: species
         integer, pointer :: chem_id(:) ! maps species to chem id
         ! index from 1 to species
         ! value is between 1 and num_chem_isos         
         integer, pointer :: net_iso(:) ! maps chem id to species number
         ! index from 1 to num_chem_isos (defined in chem_def)
         ! value is 0 if the iso is not in the current net
         ! else is value between 1 and number of species in current net
         real(dp), intent(in) :: xa(:) ! mass fractions

         ! OUTPUT
         real(dp), intent(out) :: frac_Type2
         real(dp), intent(out) :: kap ! opacity
         real(dp), intent(out) :: dln_kap_dlnRho ! partial derivative at constant T
         real(dp), intent(out) :: dln_kap_dlnT   ! partial derivative at constant Rho
         integer, intent(out) :: ierr ! 0 means AOK.

         real(dp) :: log10_R

         real(dp) :: w_kap, w_dln_kap_dlnRho, w_dln_kap_dlnT
         real(dp) :: c_kap, c_dln_kap_dlnRho, c_dln_kap_dlnT
         real(dp) :: alfa, beta, logT, lower_bdy, upper_bdy
         real(dp) :: d_alfa_dlnT

         logical :: dbg = .false.
         real(dp), parameter :: kap_min = 1e-6

         type (star_info), pointer :: s
         call get_star_ptr(id, s, ierr)

         ! aliases
         logT = log10_T
         lower_bdy = logT_blend - delta_logT_blend
         upper_bdy = logT_blend + delta_logT_blend

         alfa = (logT - lower_bdy) / (upper_bdy - lower_bdy)
         alfa = max(min(alfa,1d0),0d0) ! force in [0,1]
         beta = 1d0 - alfa

         if (logT >= lower_bdy) then
            call kap_get( &
                 handle, zbar, X, Z, Zbase, XC, XN, XO, XNe, &
                 log10_rho, log10_T, &
                 lnfree_e, d_lnfree_e_dlnRho, d_lnfree_e_dlnT, &
                 frac_Type2, w_kap, w_dln_kap_dlnRho, w_dln_kap_dlnT, ierr)
         else
            w_kap = 0
            w_dln_kap_dlnRho = 0
            w_dln_kap_dlnT = 0
         end if

         if (logT <= upper_bdy) then

            call kasen_CO(log10_rho, log10_T, &
                 c_kap, c_dln_kap_dlnRho, c_dln_kap_dlnT, ierr)

            if (ierr .lt. 0) then 
               write(*,*) "k =", k
               write(*,*) "X =", X
               write(*,*) "Z =", Z
               write(*,*) "Zbase =", Zbase
               write(*,*) "XC =", XC
               write(*,*) "XN =", XN
               write(*,*) "XO =", XO
               write(*,*) "XNe =", XNe
               write(*,*) "log10_rho =", log10_rho
               write(*,*) "log10_T =", log10_T

               write(*,*) "c_kap =", c_kap
               write(*,*) "c_dln_kap_dlnRho=", c_dln_kap_dlnRho
               write(*,*) "c_dln_kap_dlnT=", c_dln_kap_dlnT

               if (dbg) then 
                  stop
               else
                  c_kap = kap_min
                  c_dln_kap_dlnRho = 0
                  c_dln_kap_dlnT = 0

               end if
            endif

         else
            c_kap = 0
            c_dln_kap_dlnRho = 0
            c_dln_kap_dlnT = 0
         end if

         kap = alfa*w_kap + beta*c_kap

         if ((alfa .eq. 0d0) .or. (alfa .eq. 1d0)) then
            d_alfa_dlnT = 0
         else
            d_alfa_dlnT = (1d0 / ln10) / (upper_bdy - lower_bdy)
         end if

         dln_kap_dlnRho = (alfa*w_kap*w_dln_kap_dlnRho + beta*c_kap*c_dln_kap_dlnRho)/kap
         dln_kap_dlnT = (alfa*w_kap*w_dln_kap_dlnT + beta*c_kap*c_dln_kap_dlnT)/kap + &
              (d_alfa_dlnT*w_kap - d_alfa_dlnT*c_kap)/kap

      end subroutine cold_CO_kap_get
